//
// Implementation file for CFileSelector.
//
#include "stdafx.h"
#include "FileSelector.h"
#include <windowsx.h>
#include <commctrl.h>


#define ID_SELECTOR_BUTTON 131
#define ID_SELECTOR_LABEL  132


//////////////////////////////////////////////////////////
// CFileSelector constructors/destructors
//////////////////////////////////////////////////////////

CFileSelector::CFileSelector()
{
    // default filter is to look for misc image files. (Including
    // .png here is false advertisement, as we don't support it.)
    TCHAR szDefFilter[] = TEXT("Image Formats(*.bmp,*.gif, *.jpg, *.png)\0*.BMP;*.GIF;*.JPG;*.PNG\0All Files (*.*)\0*.*\0");

    // Let's copy it so that we don't need to 
    m_filter = new TCHAR[sizeof szDefFilter + 1];
    _tcsncpy(m_filter, szDefFilter, sizeof(szDefFilter)+1);
    m_isVertical = false;
}

CFileSelector::~CFileSelector()
{
    delete m_filter;
    delete m_edit;
    delete m_btn;
}


HWND
CFileSelector::Create
		( HWND     hWndParent
		, LPCTSTR  label
		, LPCTSTR  defValue
		, int nWidth, int nHeight
		, int selectorID
		)
{
    HWND  hWndButton, hWndEdit;
    RECT r;
    SIZE* pSize;

    r.top = r.left = 0;
    r.bottom = nHeight;
    r.right = nWidth - 6;

    CBox::Create(label, 150,50, hWndParent);

    m_edit = new CEditBox();
    hWndEdit = m_edit->Create(*this, r, defValue,0,0, ID_SELECTOR_LABEL);

    r.top = r.left = 0;
    r.bottom = 20;
    r.right = 20;

    m_btn = new CBtn();
    hWndButton = m_btn->Create(*this, r, _T("..."), 0,0, ID_SELECTOR_BUTTON);

    // indicate that neither the edit control nor the button should
    // be resized vertically by the box.
    SetProp(hWndEdit, RESIZE_PROPERTY_VERT,   (HANDLE)RESIZE_NONE);
    SetProp(hWndButton, RESIZE_PROPERTY_VERT, (HANDLE)RESIZE_NONE);
    SetProp(hWndEdit, RESIZE_PROPERTY_HORIZ,  (HANDLE)RESIZE_NORMAL);
  
    //
    // Store original sizes so that we can resize correctly later on.
    //

    pSize = (SIZE*)malloc(sizeof(SIZE));
    if (pSize) {
        RECT r;

        ::GetWindowRect(hWndEdit, &r);

        pSize->cx = r.right - r.left;
        pSize->cy = r.bottom - r.top;
        SetProp(hWndEdit, RESIZE_PROPERTY_ORIGSIZE, (HANDLE)pSize);
    }
    
    // Finished adding components, lay them out.
    CalcLayout();

    return m_hWnd;
}

// 
// FUNCTION: SetFilter(const char*)
//
// PURPOSE:  Set the filter categories to use in the Open-File dialog.
//
// COMMENTS: Copies the filter string, so resp. for freeing that string
//           doesn't transfer over to the CFileSelector object.
//
void
CFileSelector::SetFilter(LPCTSTR f, unsigned int len)
{
    if (_tcslen(m_filter) < len) {
        delete m_filter;
        m_filter = new TCHAR[len+1];
    }
    _tcsncpy(m_filter,f,len+1); 
}


LRESULT
CFileSelector::OnGetText(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
    return ::SendMessage(GetDlgItem(ID_SELECTOR_LABEL), uMsg, wParam, lParam);
}


void CFileSelector::SetFileName(LPCTSTR str)
{
    ::Edit_SetText(GetDlgItem(ID_SELECTOR_LABEL), str);
}

LRESULT
CFileSelector::OnClick(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	// Time to pop up the FileOpen dialog.
    OPENFILENAME ofn;
    TCHAR pstr[MAX_PATH];
	TCHAR  imgStr[255], imageDir[255];
    LPTSTR dirStr;
	int i = 0;
	int isThere;
    HKEY panKey;
	DWORD dirStrLen = 254;
	LONG lRes;

    imageDir[0] = '\0';

    ::Edit_GetText(GetDlgItem(ID_SELECTOR_LABEL), imgStr, 255);
	  
	isThere = GetFileAttributes(imgStr);
	  
	// Convert forward to backward slashes; Path* funs seem only to
	// like the latter
	while (imgStr[i] != '\0') {
	  if (imgStr[i] == '/') {
	    imgStr[i] = '\\';
	  }
	  i++;
	}
	  
	// Get at the dirname
	PathRemoveFileSpec(imgStr);


	// Open our (machine-wide) location in the registry:
	lRes = RegOpenKeyEx
	            ( HKEY_LOCAL_MACHINE
                , TEXT("Software\\Pan")
		        , 0
		        , KEY_READ
		        , &panKey
		        );
    if ( lRes == ERROR_SUCCESS ) {
	   // Look for "imageDir" string value
	   lRes = RegQueryValueEx ( panKey
				              , TEXT("imageDir")
			                  , NULL
			                  , NULL
			                  , (BYTE*)imageDir
			                  , &dirStrLen
			                  );

	    RegCloseKey( panKey );
	}


	  
	// 
	// if there's no dirname *and* the current filename can't be found,
	// use "imageDir" in the Registry as the initial value.
	//
	if ((imgStr == NULL || _tcslen(imgStr) == 0) && isThere == (-1)) {
        dirStr = imageDir;
    } else {
        dirStr = imgStr;
    }
	    
	// This *has* to be zero-terminated before passing it in
	pstr[0] = TEXT('\0');
	  
	ofn.lStructSize       = sizeof(OPENFILENAME);
	ofn.hwndOwner         = GetDesktopWindow();
	ofn.lpstrFilter       = m_filter;
	ofn.lpstrCustomFilter = NULL;
	ofn.nFilterIndex      = 1;
	ofn.lpstrFile         = (LPTSTR)pstr;
	ofn.nMaxFile          = MAX_PATH;
	ofn.lpstrInitialDir   = dirStr;
	ofn.lpstrTitle        = NULL;
    ofn.lpstrFileTitle    = NULL;
	ofn.lpstrDefExt       = NULL;
	ofn.Flags             = OFN_FILEMUSTEXIST | OFN_READONLY | OFN_PATHMUSTEXIST;

	// We're now ready to ask for the selection:
    if ( !GetOpenFileName(&ofn) ) {
        TCHAR		sErrMsg[256];
	    long		errMsg;
	    
	    errMsg = CommDlgExtendedError();
	    
	    if (errMsg == 0) { // Did the user Cancel?
	      return 0;
	    } else {
	      wsprintf(sErrMsg,TEXT("Error %d returned from GetOpenFileName()"), errMsg);
	      MessageBox(sErrMsg, TEXT("SelectFile"), MB_OK);
	      return 0;
	    } 
	} else {

        if (_tcslen(imageDir) > 0 && (_tcsstr(pstr, imageDir) == pstr)) {
           // If the user selected a file from 'imageDir', leave out
           // the 'imageDir' from the string we report in the edit box.
           // 
           PathStripPath(pstr);
        }
        ::Edit_SetText(GetDlgItem(ID_SELECTOR_LABEL), pstr);

        // notify parent that selection has changed.
	    SendMessage ( m_hWnd
			        , WM_COMMAND
			        , MAKELONG(GetWindowLong(GWL_ID), FS_CHANGED)
			        , (LPARAM)m_hWnd
			        );
	    SendMessage ( GetParent()
			        , WM_COMMAND
			        , MAKELONG(GetWindowLong(GWL_ID), FS_CHANGED)
			        , (LPARAM)m_hWnd
			        );
	    return 0;
	}
}


LRESULT
CFileSelector::OnChange(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
    // Just notify parent that selection has changed.
	SendMessage ( GetParent()
		        , WM_COMMAND
		        , MAKELONG(GetWindowLong(GWL_ID), FS_CHANGED)
		        , (LPARAM)m_hWnd
		        );
	SendMessage ( m_hWnd
		        , WM_COMMAND
		        , MAKELONG(GetWindowLong(GWL_ID), FS_CHANGED)
		        , (LPARAM)m_hWnd
		        );
	return 0;
}
